/* sht1x.h
 * basic 1-wire slave functions
 *
 * Copyright (C) 2011 Bartek Fabiszewski (www.fabiszewski.net)
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SHT1X_H_
#define SHT1X_H_

#include <math.h> // for dew point calculations

typedef union {
	unsigned int i;
	float f;
} value;

//----------------------------------------------------------------------------------
// modul-var
//----------------------------------------------------------------------------------
enum {
	TEMP, HUMI
};

#define noACK 0
#define ACK   1
							//adr  command  r/w
#define STATUS_REG_W 0x06   //000   0011    0
#define STATUS_REG_R 0x07   //000   0011    1
#define MEASURE_TEMP 0x03   //000   0001    1
#define MEASURE_HUMI 0x05   //000   0010    1
#define RESET        0x1e   //000   1111    0

#define SENSOR_TYPE	 0x1    // sht1x, sht7x

char sht1x_write_byte(byte value);
char sht1x_read_byte(byte ack);
void sht1x_transstart(void);
void sht1x_connectionreset(void);
char sht1x_softreset(void);
char sht1x_read_statusreg(byte *p_value, byte *p_checksum);
char sht1x_write_statusreg(byte *p_value);
char sht1x_measure(byte *p_value, byte *p_checksum, byte mode);
char sht1x_measure_buf(byte *p_buffer, byte mode);
void sht1x_read_buf(void);
void calc_sth11(float *p_humidity, float *p_temperature);
float calc_dewpoint(float h, float t);

#endif /* SHT1X_H_ */
