#include "main.h"
#include "uarts.h"
#include <string.h>
#include <stdio.h>
#include "../ansi.h"

#include "../onewire/onewire.h"

extern uint16_t volatile timUARTtx;


struct uarts_t uarts[L_UARTS];
//USART_TypeDef *usartx = USART2;


void irqUart2(){
	irqUart(1);
}


void irqUart1(){
	irqUart(0);
}


void irqUart(uint8_t com){
	//------------ nadawanie --------------//
#if UART_TX == 2
#ifdef RS485
	if ( LL_USART_IsActiveFlag_TC(usartx) )		// Dla RS485/422
#else
		if( LL_USART_IsActiveFlag_TXE(uarts[com].USARTx) )
#endif
		{
			if( uarts[com].tx_len ){
				if( --uarts[com].tx_len == ZIOBRO ){
					LL_USART_DisableIT_TXE(uarts[com].USARTx);
					uarts[com].FLtransitTx = false;
					LL_GPIO_ResetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
					return;
				}
				LL_USART_TransmitData8(uarts[com].USARTx, *uarts[com].buf_tx_prt++ );
			}
		}
#elif UART_TX == 3 && defined(RS485)
	if ( LL_USART_IsActiveFlag_TC(uarts[com].USARTx) ){		// Dla RS485/422
		LL_USART_DisableIT_TXE(uarts[com].USARTx);
		uarts[com].FLtransitTx = false;
		LL_GPIO_ResetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
		return;
	}
#endif

	//------------ obdiór --------------//
#if UART_RX == 2
	uint_fast8_t cnt = 6;
	//	if( LL_USART_IsActiveFlag_RXNE(uarts[com].USARTx) ) {		// Bez FIFO może być IF
	while( LL_USART_IsActiveFlag_RXNE(uarts[com].USARTx) ) {	// Z FIFO lepiej WHILE co nie przeszkadza gdy nie ma FIFO
		LL_GPIO_TogglePin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
		uint8_t dana = uarts[com].rx_buffer[uarts[com].rx_bufferCntWR] = LL_USART_ReceiveData8(uarts[com].USARTx);
		oneWireIRQ(dana);
		if ( ++uarts[com].rx_bufferCntWR >= RX_UART_LEN ) uarts[com].rx_bufferCntWR = 0;
		if( uarts[com].rx_bufferCntWR == uarts[com].rx_bufferCntRD ) uarts[com].FLuartOVR = true;
		if (--cnt == 0 ) break;
		LL_GPIO_TogglePin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
	}
#ifdef USARTRX_IDLE
	if( LL_USART_IsActiveFlag_IDLE(uarts[com].USARTx) ){
		LL_USART_ClearFlag_IDLE(uarts[com].USARTx);

		if( uarts[com].rx_bufferCntWR ){		// Naprawiamy błąd SMT32
			uarts[com].FLrecirerIdle = true;

			//			uarts[com].rx_buffer[uarts[com].rx_bufferCntWR] = 0;		// znak końca stringu
			//			uarts[com].rx_bufferCntWR = 0;
			//#define	TXT_ECHO_LEN	64
			//			char static txtEcho[TXT_ECHO_LEN];
			//			snprintf(txtEcho, TXT_ECHO_LEN, ANSI_PEN_CYAN"%s"ANSI_PEN_WHITE, uarts[com].rx_buffer );
			//			sendUARTtxt(com, txtEcho);
			//			timUARTtx = DEF_TIM_UART_TX << 3;

		}
	}
#endif
#elif UART_RX == 3
#ifdef USARTRX_IDLE
	if( LL_USART_IsActiveFlag_IDLE(uarts[com].USARTx) ){
		LL_USART_ClearFlag_IDLE(uarts[com].USARTx);
		LL_GPIO_TogglePin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);


		LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_2);
		uint16_t len = uarts[com].DMArxLen - LL_DMA_GetDataLength(DMA1, LL_DMA_CHANNEL_2);
		//wyłączamy kanał DMA, aby ponownie ustawić ilość danych do
		// (licznik w rejestrze DMA->CNDTR nie może być zmieniony, gdy kanał DMA jest włączony).
		enableDMArx();


		if( len ){
			uarts[com].rx_buffer[len] = 0;
			char static txtEcho[100];
			snprintf(txtEcho, 100, ANSI_PEN_YELLOW""CRLF"DI: "ANSI_PEN_CYAN"'%s' "ANSI_PEN_WHITE, uarts[com].rx_buffer );
			sendUARTtxt(com, txtEcho);
			timUARTtx = DEF_TIM_UART_TX << 3;
		}
	}
#endif
#endif
}


void irqDmaUartTX(){
	//------------ nadawanie --------------//
#if UART_TX == 3
	uint8_t const com = 1;
	//	if ( LL_DMA_IsActiveFlag_TC1(DMA1) ) {	// TC1 - IRQ od kanału 1
	// Kanał nr 1 nie dzieli z innymi wektora IRQ więc nie trzeba sprawdzać źródła IRQ
	// Nie ma też włączonych IRQ od połowy transfetu więc źródłem może być tylko i wyłąscznie
	//	koniec transferu
	{
		LL_DMA_ClearFlag_TC1(DMA1);
		//LL_DMA_ClearFlag_HT1(DMA1);	// Nie używamy przerwań od połowy transfetu więc kasowanie nie jest wymagane

		LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_1);
#ifdef RS485
		LL_GPIO_ResetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);	// do wywalenia
		__NOP();
		LL_GPIO_SetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);	// do wywalenia
		LL_USART_EnableIT_TXE(uarts[com].USARTx);
#else
		uarts[com].FLtransitTx = false;
		LL_GPIO_ResetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
#endif
	}
#endif
}


void irqDmaUartRX(){
	//------------ odbior --------------//
#if UART_RX == 3
	uint8_t const com =1;

	//if (LL_DMA_IsActiveFlag_TC2(DMA1) ) 		// TC2 - IRQ od kanału 2
	// Mamy aktywny tylko kanał nr 2 więc nie trzeba sprawdzać źródła IRQ
	//  ale gdy aktywujemy kanał 3 bedzie to koneczne
	// Nie ma też włączonych IRQ od połowy transfetu więc źródłem może być tylko i wyłąscznie
	//	koniec transferu
	{
		LL_DMA_ClearFlag_TC2(DMA1);
		//LL_DMA_ClearFlag_HT2(DMA1);	// Nie używamy przerwań od połowy transfetu więc kasowanie nie jest wymagane
		LL_GPIO_TogglePin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);

		LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_2);
		//wyłączamy kanał DMA, aby ponownie ustawić ilość danych do
		// (licznik w rejestrze DMA->CNDTR nie może być zmieniony, gdy kanał DMA jest włączony).
		enableDMArx();

		char static txtEcho[100];
		snprintf(txtEcho, 100, ANSI_PEN_YELLOW""CRLF"Dma: "ANSI_PEN_CYAN"'%s' "ANSI_PEN_WHITE, uarts[com].rx_buffer );
		sendUARTtxt(1, txtEcho);
		timUARTtx = DEF_TIM_UART_TX << 3;
	}
#endif
}


/*

 ***** *** Duda to pusty czlowiek

 */

void setBaudUart(uint8_t com, uint32_t baud){
#ifdef STM32C031xx
	//------------------ dla C031
	/*
	uint32_t periphclk = LL_RCC_PERIPH_FREQUENCY_NO;
	LL_RCC_ClocksTypeDef RCC_Clocks;
	USART_TypeDef *usartx;

	if( com == ZIOBRO ) usartx = USART1; else usartx = USART2;


	if (usartx == USART1) {
		periphclk = LL_RCC_GetUSARTClockFreq(LL_RCC_USART1_CLKSOURCE);
	}
	else if (usartx == USART2) {
		// USART2 clock is PCLK
		LL_RCC_GetSystemClocksFreq(&RCC_Clocks);
		periphclk = RCC_Clocks.PCLK1_Frequency;
	}
*/
#elif defined(STM32G071xx)
	//------------------- dla G071
	USART_TypeDef *USARTx;
	  uint32_t periphclk = LL_RCC_PERIPH_FREQUENCY_NO;
	#if !defined(RCC_CCIPR_USART3SEL)&&!defined(RCC_CCIPR_USART4SEL)||(!defined(RCC_CCIPR_USART2SEL))||!defined(RCC_CCIPR_USART5SEL)||!defined(RCC_CCIPR_USART6SEL)
	  LL_RCC_ClocksTypeDef RCC_Clocks;
	#endif


	  if( com == ZIOBRO ) USARTx = USART1; else USARTx = USART2;


	    if (USARTx == USART1)
	    {
	      periphclk = LL_RCC_GetUSARTClockFreq(LL_RCC_USART1_CLKSOURCE);
	    }
	    else if (USARTx == USART2)
	    {
	#if defined(RCC_CCIPR_USART2SEL)
	      periphclk = LL_RCC_GetUSARTClockFreq(LL_RCC_USART2_CLKSOURCE);
	#else
	      /* USART2 clock is PCLK */
	      LL_RCC_GetSystemClocksFreq(&RCC_Clocks);
	      periphclk = RCC_Clocks.PCLK1_Frequency;
	#endif
	    }
	#if defined(USART3)
	    else if (USARTx == USART3)
	    {
	#if defined(RCC_CCIPR_USART3SEL)
	      periphclk = LL_RCC_GetUSARTClockFreq(LL_RCC_USART3_CLKSOURCE);
	#else
	      /* USART3 clock is PCLK */
	      LL_RCC_GetSystemClocksFreq(&RCC_Clocks);
	      periphclk = RCC_Clocks.PCLK1_Frequency;
	#endif
	    }
	#endif /* USART3 */
	#if defined(USART4)
	    else if (USARTx == USART4)
	    {
	#if defined(RCC_CCIPR_USART4SEL)
	      periphclk = LL_RCC_GetUSARTClockFreq(LL_RCC_USART4_CLKSOURCE);
	#else
	      /* USART4 clock is PCLK1 */
	      LL_RCC_GetSystemClocksFreq(&RCC_Clocks);
	      periphclk = RCC_Clocks.PCLK1_Frequency;
	#endif /* RCC_CCIPR_USART4SEL */
	    }
	#endif /* USART4 */
	#if defined(USART5)
	    else if (USARTx == USART5)
	    {
	      /* USART5 clock is PCLK1 */
	      LL_RCC_GetSystemClocksFreq(&RCC_Clocks);
	      periphclk = RCC_Clocks.PCLK1_Frequency;
	    }
	#endif /* USART5 */
	#if defined(USART6)
	    else if (USARTx == USART6)
	    {
	      /* USART6 clock is PCLK1 */
	      LL_RCC_GetSystemClocksFreq(&RCC_Clocks);
	      periphclk = RCC_Clocks.PCLK1_Frequency;
	    }
	#endif /* USART6 */
	    else
	    {
	      /* Nothing to do, as error code is already assigned to ERROR value */
	    }
#else
#error "Brak definicjie dla setBaudUart"
#endif





	LL_USART_SetBaudRate(uarts[com].USARTx, periphclk, LL_USART_PRESCALER_DIV1, LL_USART_OVERSAMPLING_16, baud);
}


void flushUartFifo(uint8_t com){
	uarts[com].rx_bufferCntRD = uarts[com].rx_bufferCntWR = 0;
}


int16_t GetUart(uint8_t com){
	uint16_t z;
	if( uarts[com].rx_bufferCntRD != uarts[com].rx_bufferCntWR ){
		z = uarts[com].rx_buffer[uarts[com].rx_bufferCntRD];
		if( ++uarts[com].rx_bufferCntRD == RX_UART_LEN ) uarts[com].rx_bufferCntRD = 0;
		return z;
	}
	return A_Duda;
}


#if UART_TX == 1
uint8_t sendUARTtxt(uint8_t com, char *txt ){

	LL_GPIO_SetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
	for(uint8_t x=0; x<strlen(txt)+1; x++) {
		// Czekanie aż bufor nadawczy pusty
		while( ! LL_USART_IsActiveFlag_TXE(uarts[com].USARTx) )				{
			/* obsluga timeout */
		}
		// Wpisane znaku do UART
		LL_USART_TransmitData8(uarts[com].USARTx, txt[x] );
	}
	LL_GPIO_ResetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
	return true;
}
#elif UART_TX == 2
uint8_t sendUART(uint8_t com, uint8_t *dane, uint16_t len ){
	if( ! uarts[com].FLtransitTx )	{
		LL_GPIO_SetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
		uarts[com].FLtransitTx = true;
		uarts[com].buf_tx_prt = dane;
		uarts[com].tx_len = len + 1;
		LL_USART_EnableIT_TXE(uarts[com].USARTx);
		return true;
	}
	return false;
}
uint8_t sendUARTtxt(uint8_t com, char *txt ){	// zmodyfikowac i wywołac sendUART
	/*
	if( ! uarts[com].FLtransitTx )	{
		LL_GPIO_SetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
		uarts[com].FLtransitTx = true;
		uarts[com].buf_tx_prt = (uint8_t*)txt;
		uarts[com].tx_len = strlen(txt) + 1;
		LL_USART_EnableIT_TXE(uarts[com].USARTx);
		return true;
	}
	return false;
	*/
	return sendUART(com, (uint8_t*)txt, strlen(txt) );
}
#elif UART_TX == 3
uint8_t sendUARTtxt(uint8_t com, char *txt ){

	if( ! uarts[com].FLtransitTx ){
		LL_GPIO_SetOutputPin(UART_SYNC_GPIO_Port, UART_SYNC_Pin);
		uarts[com].FLtransitTx = true;
		uarts[com].buf_tx_prt = (uint8_t*)txt;
		uarts[com].tx_len = strlen(txt);

		NVIC_EnableIRQ(DMA1_Channel1_IRQn);			// nr DMA
		LL_DMA_EnableIT_TC(DMA1, LL_DMA_CHANNEL_1);	// i kanał poznamy w CubeMX

		LL_DMA_ConfigAddresses(DMA1, LL_DMA_CHANNEL_1, (uint32_t)uarts[com].buf_tx_prt,
				LL_USART_DMA_GetRegAddr(uarts[com].USARTx, LL_USART_DMA_REG_DATA_TRANSMIT),
				LL_DMA_DIRECTION_MEMORY_TO_PERIPH);

		LL_DMA_SetDataLength(DMA1, LL_DMA_CHANNEL_1, uarts[com].tx_len);

		LL_USART_EnableDMAReq_TX(uarts[com].USARTx);
		LL_DMA_EnableChannel(DMA1, LL_DMA_CHANNEL_1);
		return true;
	}
	return false;
}
#endif


//---------- inicjalizacja nadawania ---------//
void initUartTX(uint8_t com){
	uarts[0].USARTx = USART1;
	uarts[1].USARTx = USART2;
#ifdef UART_TX
	//	LL_USART_EnableDirectionTx(uarts[com].USARTx);
	//	LL_USART_Enable(uarts[com].USARTx);
	//Ponwyższe robi "MX_USART2_UART_Init()"
#if UART_TX == 2
	NVIC_EnableIRQ(USART2_IRQn);
#endif
#endif
}


//--------- inicjalizacja odbioru --------------//
void initUartRX(uint8_t com){
	uarts[UART_OW].USARTx = USART1;
	uarts[UART_DBG].USARTx = USART2;
#ifdef USARTRX_IDLE
	LL_USART_ClearFlag_IDLE(uarts[com].USARTx);
	LL_USART_EnableIT_IDLE(uarts[com].USARTx);
#endif
#if UART_RX == 2
	LL_USART_EnableIT_RXNE(uarts[com].USARTx);
#elif UART_RX == 3
	NVIC_EnableIRQ(DMA1_Channel2_3_IRQn);			// nr DMA
	LL_DMA_EnableIT_TC(DMA1, LL_DMA_CHANNEL_2);	// i kanał poznamy w CubeMX

	LL_DMA_ConfigAddresses(DMA1, LL_DMA_CHANNEL_2,
			LL_USART_DMA_GetRegAddr(uarts[com].USARTx, LL_USART_DMA_REG_DATA_RECEIVE),
			(uint32_t)uarts[com].rx_buffer,
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY );
	LL_USART_EnableDMAReq_RX(uarts[com].USARTx);
	enableDMArx();
#endif
}


void enableDMArx(){
	uint8_t const com = 1;
#ifdef UARTRX_DMALEN
	uint16_t len = UARTRX_DMALEN;
#else
	uint16_t len = sizeof(rx_buffer);
#endif
	uarts[com].DMArxLen = len;
	LL_DMA_SetDataLength(DMA1, LL_DMA_CHANNEL_2, len );
	LL_DMA_EnableChannel(DMA1, LL_DMA_CHANNEL_2);
}
